#!/bin/sh
#
# MFN-100 Switch initialization script.
#
# Revision:
#   26-Jul-2017 (SZU) []: Initial version.
#

MGMT_IP_ADDR=172.16.0.27
MGMT_NETMASK=255.255.0.0

#FLX_KO_PATH=/lib/modules/$(uname -r)/extra

DATA_PART_DEV=/dev/mmcblk0p4
DATA_PART_MNT_PATH=/media/data

UD_FOLDER=_MFN-100-SW_
VERSION_FILE=/home/root/mfn-100-sw-version.info

LED_F_G=480
LED_F_R=481
LED_S_G=475
LED_S_R=483
LED_BG=434

LED_CTRL_FILE=/var/local/led_ctrl
LED_CTRL=$(if [ -e $LED_CTRL_FILE ]; then cat $LED_CTRL_FILE; else echo "1";fi)

# $1: GPIO number, $2: value to be set
set_led() {
 if [ "$LED_CTRL" -eq "1" ]; then
  echo "$2" > /sys/class/gpio/gpio$1/value
 fi
}

# $1: file to write version information to
create_version_file() {
 mv ${1}.base $1
 echo "" >> $1

 echo "                           --- PTP Stack ---" >> $1
 echo "" >> $1
 ptp -h | sed -n 1,2p >> $1
 echo "" >> $1
 echo "" >> $1

 echo "                          --- DE-IP Driver ---" >> $1
 echo "" >> $1
 modinfo deipce >> $1
 echo "" >> $1
 echo "" >> $1

 echo "                            --- DE-IP ---" >> $1
 echo "" >> $1
 echo "FRS:" >> $1
 echo "" >> $1
 head -6 /proc/driver/deipce/device00_common_registers >> $1
 echo "" >> $1

 echo "FRTC0:" >> $1
 head -5 /proc/driver/flx_frtc/device00_registers >> $1
 echo "" >> $1
 echo "FRTC1:" >> $1
 head -5 /proc/driver/flx_frtc/device01_registers >> $1

 echo "" >> $1
}

init_interfaces() {
 ip link add name deipce0 type bridge
 ip link set dev deipce0 up
 ip link set dev IE01 master deipce0 up
 ip link set dev CE01 master deipce0 up
 ip link set dev CE02 master deipce0 up
 ip link set dev CE03 master deipce0 up
 ip link set dev CE04 master deipce0 up
 ip link set dev SFP master deipce0 up
 ip link set dev COMe master deipce0 up

 ifconfig SE01 $MGMT_IP_ADDR netmask $MGMT_NETMASK
}

shutdown_interfaces() {
 ip link set dev deipce0 down
 ip link delete dev deipce0

 ip link set dev SE01 down
 ip link delete dev SE01
}

start() {
 # mount data partition
 if [ ! -d $DATA_PART_MNT_PATH ]; then
  mkdir -p $DATA_PART_MNT_PATH
 fi
 mount $DATA_PART_DEV $DATA_PART_MNT_PATH

 # delete the update folder if available
 if [ -d $DATA_PART_MNT_PATH/$UD_FOLDER ];then
  echo "MFN-100-SW update found, deleting..."
  rm -rf $DATA_PART_MNT_PATH/$UD_FOLDER
  rm -f $DATA_PART_MNT_PATH/$VERSION_FILE
 fi

 # bring up eth0
 ip link set eth0 up

 # load kernel modules
 modprobe flx_eth_mdio
 modprobe deipce port_ts=1

 # bring up DE-IP interfaces
 init_interfaces

 # export GPIO for STATUS and FUNCTION LEDs
 echo "$LED_F_G" > /sys/class/gpio/export
 echo "$LED_F_R" > /sys/class/gpio/export
 echo "$LED_S_G" > /sys/class/gpio/export
 echo "$LED_S_R" > /sys/class/gpio/export
 echo "$LED_BG" > /sys/class/gpio/export

 echo "out" > /sys/class/gpio/gpio$LED_F_G/direction
 echo "out" > /sys/class/gpio/gpio$LED_F_R/direction
 echo "out" > /sys/class/gpio/gpio$LED_S_G/direction
 echo "out" > /sys/class/gpio/gpio$LED_S_R/direction
 echo "out" > /sys/class/gpio/gpio$LED_BG/direction

 # Create a version information file if none exists
 if [ ! -f $VERSION_FILE ]; then
  create_version_file $VERSION_FILE
 fi

 # start monitoring for updates
 #(/etc/mfn-switch-update.sh) &

 # start monitoring for configuration script
 (/etc/mfn-switch-cfg.sh) &

 # signal initialization complete
 set_led $LED_F_R 0
 set_led $LED_F_G 1
}

stop() {

 # FUNCTION gets orange to signal shutdown/reboot
 set_led $LED_F_G 1
 set_led $LED_F_R 1

 set_led $LED_S_G 0
 set_led $LED_S_R 0

 shutdown_interfaces
 
 rmmod deipce
 rmmod flx_eth_mdio

 umount $DATA_PART_MNT_PATH
}

uninstall() {
 echo "MFN-Init: Nothing to do for uninstall..."
}

case "$1" in
  start)
    start
    ;;
  stop)
    stop
    ;;
  uninstall)
    uninstall
    ;;
  retart)
    stop
    start
    ;;
  *)
    echo "Usage: $0 {start|stop|restart|uninstall}"
esac

